{% load accessors json_templates %}
<table>
    <tbody>
    {% for code, lang_name in widget.languages %}
        <tr>
            <td>{{ lang_name }}</td>
            <td>
                <textarea id="id_{{ widget.name }}__{{ code }}" name="{{ widget.name }}__{{ code }}">{{ widget.value|dict_get:code }}</textarea>
            </td>
        </tr>
    {% endfor %}
    </tbody>
    {% with widget.value|dict_get:"_meta" as meta %}
    {% if meta %}
    <input type="hidden" name="{{widget.name}}___meta" value="{{ meta|json_encode }}" />
    {% endif %}
    {% endwith %}
</table>
