# Generated by Django 4.2.11 on 2024-04-16 06:12

import django.db.models.deletion
from django.db import migrations, models

import bx_django_utils.data_types.gtin.model_fields
import bx_django_utils.data_types.gtin.validators
import bx_django_utils.models.color_field
import bx_django_utils.translation


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name='RawTranslatedModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('translated', models.JSONField()),
                ('translated_multiline', models.JSONField(blank=True, default=dict)),
                ('not_translated', models.TextField(default='A Default Value')),
            ],
            options={
                'db_table': 'test_app_translatedmodel',
                'managed': False,
            },
        ),
        migrations.CreateModel(
            name='ColorFieldTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('required_color', bx_django_utils.models.color_field.ColorModelField(max_length=7)),
                (
                    'optional_color',
                    bx_django_utils.models.color_field.ColorModelField(blank=True, max_length=7, null=True),
                ),
            ],
        ),
        migrations.CreateModel(
            name='ConnectedUniqueSlugModel1',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'translated',
                    bx_django_utils.translation.TranslationField(
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                    ),
                ),
                (
                    'translated_slug',
                    bx_django_utils.translation.TranslationSlugField(
                        additional_uniqueness=(
                            {
                                'app_label': 'test_app',
                                'field_name': 'translated_slug',
                                'model_name': 'ConnectedUniqueSlugModel2',
                            },
                        ),
                        blank=True,
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                        unique=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='ConnectedUniqueSlugModel2',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'translated',
                    bx_django_utils.translation.TranslationField(
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                    ),
                ),
                (
                    'translated_slug',
                    bx_django_utils.translation.TranslationSlugField(
                        additional_uniqueness=(
                            {
                                'app_label': 'test_app',
                                'field_name': 'translated_slug',
                                'model_name': 'ConnectedUniqueSlugModel1',
                            },
                        ),
                        blank=True,
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                        unique=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='GtinFieldTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'default_gtin',
                    bx_django_utils.data_types.gtin.model_fields.GtinModelField(
                        max_length=14,
                        validators=[
                            bx_django_utils.data_types.gtin.validators.GtinValidator(accepted_length=(12, 13, 14))
                        ],
                        verbose_name='GTIN',
                    ),
                ),
                (
                    'all_gtin',
                    bx_django_utils.data_types.gtin.model_fields.GtinModelField(
                        blank=True,
                        max_length=14,
                        validators=[
                            bx_django_utils.data_types.gtin.validators.GtinValidator(accepted_length=(12, 13, 14))
                        ],
                        verbose_name='GTIN',
                    ),
                ),
                (
                    'ean13',
                    bx_django_utils.data_types.gtin.model_fields.GtinModelField(
                        blank=True,
                        max_length=14,
                        null=True,
                        validators=[
                            bx_django_utils.data_types.gtin.validators.GtinValidator(accepted_length=(12, 13, 14))
                        ],
                        verbose_name='GTIN',
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='NonUniqueTranslatedSlugTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'translated',
                    bx_django_utils.translation.TranslationField(
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                    ),
                ),
                (
                    'translated_slug',
                    bx_django_utils.translation.TranslationSlugField(
                        additional_uniqueness=None,
                        blank=True,
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                        unique=False,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='PolymorphicVehicle',
            fields=[
                ('license_plate', models.CharField(max_length=64, primary_key=True, serialize=False)),
                ('color', models.TextField()),
                (
                    'polymorphic_ctype',
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='polymorphic_%(app_label)s.%(class)s_set+',
                        to='contenttypes.contenttype',
                    ),
                ),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
        ),
        migrations.CreateModel(
            name='StoreSaveModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64)),
            ],
        ),
        migrations.CreateModel(
            name='TimetrackingTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'create_dt',
                    models.DateTimeField(
                        blank=True,
                        editable=False,
                        help_text='ModelTimetrackingMixin.create_dt.help_text',
                        null=True,
                        verbose_name='ModelTimetrackingMixin.create_dt.verbose_name',
                    ),
                ),
                (
                    'update_dt',
                    models.DateTimeField(
                        blank=True,
                        editable=False,
                        help_text='ModelTimetrackingMixin.update_dt.help_text',
                        null=True,
                        verbose_name='ModelTimetrackingMixin.update_dt.verbose_name',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='TranslatedModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'translated',
                    bx_django_utils.translation.TranslationField(
                        languages=(('de-de', 'de-de'), ('en-us', 'en-us'), ('es', 'es')),
                        max_value_length=None,
                        min_value_length=None,
                    ),
                ),
                (
                    'translated_multiline',
                    bx_django_utils.translation.TranslationField(
                        blank=True,
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                    ),
                ),
                ('not_translated', models.TextField(default='A Default Value')),
            ],
        ),
        migrations.CreateModel(
            name='TranslatedSlugTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'translated',
                    bx_django_utils.translation.TranslationField(
                        languages=(('de-de', 'German'), ('en-us', 'US-English'), ('es', 'Spanish')),
                        max_value_length=None,
                        min_value_length=None,
                    ),
                ),
                (
                    'translated_slug',
                    bx_django_utils.translation.TranslationSlugField(
                        additional_uniqueness=None,
                        blank=True,
                        languages=(('de-de', 'de-de'), ('en-us', 'en-us'), ('es', 'es')),
                        max_value_length=None,
                        min_value_length=None,
                        unique=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='ValidateLengthTranslations',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'translated',
                    bx_django_utils.translation.TranslationField(
                        languages=(('de', 'German'), ('en', 'English')), max_value_length=20, min_value_length=3
                    ),
                ),
                (
                    'translated_slug',
                    bx_django_utils.translation.TranslationSlugField(
                        additional_uniqueness=None,
                        blank=True,
                        languages=(('de', 'German'), ('en', 'English')),
                        max_value_length=20,
                        min_value_length=3,
                        unique=True,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name='PolymorphicBike',
            fields=[
                (
                    'polymorphicvehicle_ptr',
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to='test_app.polymorphicvehicle',
                    ),
                ),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('test_app.polymorphicvehicle',),
        ),
        migrations.CreateModel(
            name='PolymorphicCar',
            fields=[
                (
                    'polymorphicvehicle_ptr',
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to='test_app.polymorphicvehicle',
                    ),
                ),
            ],
            options={
                'abstract': False,
                'base_manager_name': 'objects',
            },
            bases=('test_app.polymorphicvehicle',),
        ),
        migrations.CreateModel(
            name='CreateOrUpdateTestModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                (
                    'create_dt',
                    models.DateTimeField(
                        blank=True,
                        editable=False,
                        help_text='ModelTimetrackingMixin.create_dt.help_text',
                        null=True,
                        verbose_name='ModelTimetrackingMixin.create_dt.verbose_name',
                    ),
                ),
                (
                    'update_dt',
                    models.DateTimeField(
                        blank=True,
                        editable=False,
                        help_text='ModelTimetrackingMixin.update_dt.help_text',
                        null=True,
                        verbose_name='ModelTimetrackingMixin.update_dt.verbose_name',
                    ),
                ),
                ('name', models.CharField(max_length=64)),
                ('slug', models.SlugField(max_length=64)),
                ('blank_field', models.CharField(blank=True, max_length=64)),
                ('null_field', models.CharField(blank=True, max_length=64, null=True)),
                ('uuid_field', models.UUIDField(blank=True, null=True)),
                (
                    'many2one_rel',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to='test_app.timetrackingtestmodel',
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
